/*
 * GrammarToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarToStringComposer.h"
#include "grammar/GrammarClasses.h"
#include "../StringApi.hpp"

namespace grammar {

template<class T>
void GrammarToStringComposer::composeCFLikeGrammar(std::ostream& output, const T& grammar) {
	bool first;

	output << " (" << std::endl;

	output << "{";
	first = false;
	for(const auto& symbol : grammar.getNonterminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = false;
	for(const auto& symbol : grammar.getTerminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{ ";
	first = true;
	for(const auto& rule : grammar.getRawRules() ) {
		if(first)
			first = false;
		else
			output << "," << std::endl << "  ";
		alib::stringApi<alphabet::Symbol>::compose(output, rule.first);
		output << " ->";
		bool innerFirst = true;
		for(const auto& rhs : rule.second) {
			if(innerFirst)
				innerFirst = false;
			else
				output << " |";
			for(const auto& symbol : rhs) {
				output << " ";
				alib::stringApi<alphabet::Symbol>::compose(output, symbol);
			}
		}
	}
	output << "}," << std::endl;
	alib::stringApi<alphabet::Symbol>::compose(output, grammar.getInitialSymbol());
	output << ")" << std::endl;
}

template<class T>
void GrammarToStringComposer::composeCSLikeGrammar(std::ostream& output, const T& grammar) {
	bool first;

	output << " (" << std::endl;

	output << "{";
	first = false;
	for(const auto& symbol : grammar.getNonterminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = false;
	for(const auto& symbol : grammar.getTerminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = true;
	for(const auto& rule : grammar.getRules() ) {
		if(first)
			first = false;
		else
			output << "," << std::endl << " ";
		for(const auto& symbol : rule.first) {
			output << " ";
			alib::stringApi<alphabet::Symbol>::compose(output, symbol);
		}
		output << " ->";
		bool innerFirst = true;
		for(const auto& rhs : rule.second) {
			if(innerFirst)
				innerFirst = false;
			else
				output << " |";
			for(const auto& symbol : rhs) {
				output << " ";
				alib::stringApi<alphabet::Symbol>::compose(output, symbol);
			}
		}
	}
	output << "}," << std::endl;
	alib::stringApi<alphabet::Symbol>::compose(output, grammar.getInitialSymbol());
	output << ")" << std::endl;
}

template<class T>
void GrammarToStringComposer::composePreservingCSLikeGrammar(std::ostream& output, const T& grammar) {
	bool first;

	output << " (" << std::endl;

	output << "{";
	first = false;
	for(const auto& symbol : grammar.getNonterminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = false;
	for(const auto& symbol : grammar.getTerminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi<alphabet::Symbol>::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = true;
	for(const auto& rule : grammar.getRules() ) {
		if(first)
			first = false;
		else
			output << "," << std::endl << " ";
		for(const auto& symbol : std::get<0>(rule.first)) {
			output << " ";
			alib::stringApi<alphabet::Symbol>::compose(output, symbol);
		}
		output << " | ";
		alib::stringApi<alphabet::Symbol>::compose(output, std::get<1>(rule.first));
		output << " |";
		for(const auto& symbol : std::get<2>(rule.first)) {
			output << " ";
			alib::stringApi<alphabet::Symbol>::compose(output, symbol);
		}
		output << " ->";
		bool innerFirst = true;
		for(const auto& rhs : rule.second) {
			if(innerFirst)
				innerFirst = false;
			else
				output << " |";
			for(const auto& symbol : rhs) {
				output << " ";
				alib::stringApi<alphabet::Symbol>::compose(output, symbol);
			}
		}
	}
	output << "}," << std::endl;
	alib::stringApi<alphabet::Symbol>::compose(output, grammar.getInitialSymbol());
	output << ")" << std::endl;
}

void GrammarToStringComposer::compose(std::ostream& output, const Grammar& grammar) {
	getInstance().dispatch(output, grammar.getData());
}

void GrammarToStringComposer::compose(std::ostream& output, const LeftLG& grammar) {
	output << "LEFT_LG";
	composeCFLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, LeftLG> GrammarToStringComposerLeftLG = GrammarToStringComposer::RegistratorWrapper<void, LeftLG>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const LeftRG& grammar) {
	output << "LEFT_RG";
	composeCFLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, LeftRG> GrammarToStringComposerLeftRG = GrammarToStringComposer::RegistratorWrapper<void, LeftRG>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const RightLG& grammar) {
	output << "RIGHT_LG";
	composeCFLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, RightLG> GrammarToStringComposerRightLG = GrammarToStringComposer::RegistratorWrapper<void, RightLG>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const RightRG& grammar) {
	output << "RIGHT_RG";
	composeCFLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, RightRG> GrammarToStringComposerRightRG = GrammarToStringComposer::RegistratorWrapper<void, RightRG>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const LG& grammar) {
	output << "LG";
	composeCFLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, LG> GrammarToStringComposerLG = GrammarToStringComposer::RegistratorWrapper<void, LG>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const CFG& grammar) {
	output << "CFG";
	composeCFLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, CFG> GrammarToStringComposerCFG = GrammarToStringComposer::RegistratorWrapper<void, CFG>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const EpsilonFreeCFG& grammar) {
	output << "EPSILON_FREE_CFG";
	composeCFLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, EpsilonFreeCFG> GrammarToStringComposerEpsilonFreeCFG = GrammarToStringComposer::RegistratorWrapper<void, EpsilonFreeCFG>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const CNF& grammar) {
	output << "CNF";
	composeCFLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, CNF> GrammarToStringComposerCNF = GrammarToStringComposer::RegistratorWrapper<void, CNF>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const GNF& grammar) {
	output << "GNF";
	composeCFLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, GNF> GrammarToStringComposerGNF = GrammarToStringComposer::RegistratorWrapper<void, GNF>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const CSG& grammar) {
	output << "CSG";
	composePreservingCSLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, CSG> GrammarToStringComposerCSG = GrammarToStringComposer::RegistratorWrapper<void, CSG>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const NonContractingGrammar& grammar) {
	output << "NON_CONTRACTING_GRAMMAR";
	composeCSLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, NonContractingGrammar> GrammarToStringComposerNonContractingGrammar = GrammarToStringComposer::RegistratorWrapper<void, NonContractingGrammar>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const ContextPreservingUnrestrictedGrammar& grammar) {
	output << "CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR";
	composePreservingCSLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, ContextPreservingUnrestrictedGrammar> GrammarToStringComposerContextPreservingUnrestructedGrammar = GrammarToStringComposer::RegistratorWrapper<void, ContextPreservingUnrestrictedGrammar>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

void GrammarToStringComposer::compose(std::ostream& output, const UnrestrictedGrammar& grammar) {
	output << "UNRESTRICTED_GRAMMAR";
	composeCSLikeGrammar(output, grammar);
}

GrammarToStringComposer::RegistratorWrapper<void, UnrestrictedGrammar> GrammarToStringComposerUnrestructedGrammar = GrammarToStringComposer::RegistratorWrapper<void, UnrestrictedGrammar>(GrammarToStringComposer::getInstance(), GrammarToStringComposer::compose);

}
