/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#include "GraphToStringComposer.h"
#include "graph/Graph.h"
#include "graph/GraphClasses.h"
#include "../StringApi.hpp"

namespace graph {

void GraphToStringComposer::compose(std::ostream &out, const Graph &graph) {
	out << "(";
	getInstance().dispatch(out, graph.getData());
	out << ")";
}

void GraphToStringComposer::compose(std::ostream& out, const DirectedGraph &graph) {
	out << "DirectedGraph:";
	composeRepresentation(out, graph.getRepresentation());
	out << ":";
	composeNodes(out, graph.getNodes());
	out << ":";
	composeDirectedEdges(out, graph.getEdges());
	out << ":";
	composeNodeValues(out, graph);
	out << ":";
	composeEdgeValues(out, graph);
}

GraphToStringComposer::RegistratorWrapper<void, DirectedGraph> GraphToStringComposerDirectedGraph = GraphToStringComposer::RegistratorWrapper<void, DirectedGraph>(GraphToStringComposer::getInstance(), GraphToStringComposer::compose);

void GraphToStringComposer::compose(std::ostream& out, const UndirectedGraph &graph) {
	out << "UndirectedGraph:";
	composeRepresentation(out, graph.getRepresentation());
	out << ":";
	composeNodes(out, graph.getNodes());
	out << ":";
	composeUndirectedEdges(out, graph.getEdges());
	out << ":";
	composeNodeValues(out, graph);
	out << ":";
	composeEdgeValues(out, graph);
}

GraphToStringComposer::RegistratorWrapper<void, UndirectedGraph> GraphToStringComposerUndirectedGraph = GraphToStringComposer::RegistratorWrapper<void, UndirectedGraph>(GraphToStringComposer::getInstance(), GraphToStringComposer::compose);

void GraphToStringComposer::composeNode(std::ostream& out, const Node &node) {
	alib::stringApi<label::Label>::compose(out, node.getName());
}

void GraphToStringComposer::composeEdge(std::ostream& out, const DirectedEdge &edge) {
	composeNode(out, edge.getFromNode());
	alib::stringApi<label::Label>::compose(out, edge.getName());
	composeNode(out, edge.getToNode());
}

void GraphToStringComposer::composeEdge(std::ostream& out, const UndirectedEdge &edge) {
	composeNode(out, edge.getFirstNode());
	alib::stringApi<label::Label>::compose(out, edge.getName());
	composeNode(out, edge.getSecondNode());
}

void GraphToStringComposer::composeRepresentation(std::ostream &out, REPRESENTATION representation) {
	out << representationToString(representation);
}

void GraphToStringComposer::composeNodes(std::ostream &out, const std::set<Node> &nodes) {
	bool first = true;

	for (const Node &node : nodes) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeNode(out, node);
	}
}

void GraphToStringComposer::composeDirectedEdges(std::ostream &out, const std::set<DirectedEdge> &edges) {
	bool first = true;

	for (const DirectedEdge &edge : edges) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeEdge(out, edge);
	}
}

void GraphToStringComposer::composeUndirectedEdges(std::ostream &out, const std::set<UndirectedEdge> &edges) {
	bool first = true;

	for (const UndirectedEdge &edge : edges) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeEdge(out, edge);
	}
}

template<typename T>
void GraphToStringComposer::composeNodeValues(std::ostream &out, const T &graph) {
	bool first = true;

	for (auto i : graph.nodeValues) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeNode(out, i.first);
		out << "=" << i.second;
	}
}

template<typename T>
void GraphToStringComposer::composeEdgeValues(std::ostream &out, const T &graph) {
	bool first = true;

	for (auto i : graph.edgeValues) {
		if (first) {
			first = false;
		} else {
			out << ",";
		}
		composeEdge(out, i.first);
		out << "=" << i.second;
	}
}

} // namespace graph
