/*
 * LabelToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelToStringComposer.h"
#include "label/PrimitiveLabel.h"
#include "label/HexavigesimalLabel.h"
#include "label/ObjectLabel.h"
#include "label/LabelSetLabel.h"
#include "label/LabelPairLabel.h"
#include "label/UniqueLabel.h"

#include "../StringApi.hpp"

namespace label {

void LabelToStringComposer::compose(std::ostream& out, const PrimitiveLabel& label) {
	alib::stringApi<primitive::Primitive>::compose(out, label.getData());
}

LabelToStringComposer::RegistratorWrapper<void, PrimitiveLabel> LabelToStringComposerPrimitiveLabel = LabelToStringComposer::RegistratorWrapper<void, PrimitiveLabel>(LabelToStringComposer::getInstance(), LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const HexavigesimalLabel& label) {
	out << (std::string) label;
}

LabelToStringComposer::RegistratorWrapper<void, HexavigesimalLabel> LabelToStringComposerHexavigesimalLabel = LabelToStringComposer::RegistratorWrapper<void, HexavigesimalLabel>(LabelToStringComposer::getInstance(), LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const ObjectLabel& label) {
	out << (std::string) label;
}

LabelToStringComposer::RegistratorWrapper<void, ObjectLabel> LabelToStringComposerObjectLabel = LabelToStringComposer::RegistratorWrapper<void, ObjectLabel>(LabelToStringComposer::getInstance(), LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const LabelSetLabel& label) {
	out << '[';
	bool first = true;
	for(const Label& innerLabel : label.getData()) {
		if(!first)
			out << ", ";
		else
			first = false;
		getInstance().dispatch(out, innerLabel.getData());
	}
	out << ']';
}

LabelToStringComposer::RegistratorWrapper<void, LabelSetLabel> LabelToStringComposerLabelSetLabel = LabelToStringComposer::RegistratorWrapper<void, LabelSetLabel>(LabelToStringComposer::getInstance(), LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const LabelPairLabel& label) {
	out << '<';
	getInstance().dispatch(out, label.getData().first.getData());
	out << ", ";
	getInstance().dispatch(out, label.getData().second.getData());
	out << '>';
}

LabelToStringComposer::RegistratorWrapper<void, LabelPairLabel> LabelToStringComposerLabelPairLabel = LabelToStringComposer::RegistratorWrapper<void, LabelPairLabel>(LabelToStringComposer::getInstance(), LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const UniqueLabel& label) {
	out << '<';
	getInstance().dispatch(out, label.getLabel().getData());
	out << ", ";
	alib::stringApi<primitive::Primitive>::compose(out, primitive::Primitive(label.getId()));
	out << '>';
}

LabelToStringComposer::RegistratorWrapper<void, UniqueLabel> LabelToStringComposerUniqueLabel = LabelToStringComposer::RegistratorWrapper<void, UniqueLabel>(LabelToStringComposer::getInstance(), LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& output, const Label& label) {
	getInstance().dispatch(output, label.getData());
}

} /* namespace label */
