/*
 * PrimitiveFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "PrimitiveFromStringParser.h"
#include "exception/AlibException.h"
#include "primitive/Primitive.h"
#include "primitive/String.h"
#include "primitive/Integer.h"
#include "primitive/Character.h"

namespace primitive {

Primitive PrimitiveFromStringParser::parsePrimitive(std::istream& input) const {
	return parsePrimitive(input, std::set<FEATURES>({FEATURES::STRING, FEATURES::CHAR, FEATURES::INTEGER}));
}

Primitive PrimitiveFromStringParser::parsePrimitive(std::istream& input, const std::set<FEATURES>& features) const {
	PrimitiveFromStringLexer::Token token = m_Lexer.next(input);
	switch(token.type) {
	case PrimitiveFromStringLexer::TokenType::TEOF:
		throw exception::AlibException("Unexpected end of file");
	case PrimitiveFromStringLexer::TokenType::STRING:
		if(!features.count(FEATURES::STRING)) throw exception::AlibException();
		return Primitive(String(token.value));
	case PrimitiveFromStringLexer::TokenType::CHAR:
		if(!features.count(FEATURES::CHAR)) throw exception::AlibException();
		return Primitive(Character(token.value[0]));
	case PrimitiveFromStringLexer::TokenType::INTEGER:
		if(!features.count(FEATURES::INTEGER)) throw exception::AlibException();
		return Primitive(Integer(std::stoi(token.value)));
	case PrimitiveFromStringLexer::TokenType::ERROR:
		break;
	}
	throw exception::AlibException("Unrecognised input \"" + token.value + "...\"");
}

} /* namespace primitive */
