/*
 * TreeToStringComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnciek
 */

#ifndef TREE_TO_STRING_COMPOSER_H_
#define TREE_TO_STRING_COMPOSER_H_

#include <ostream>
#include <common/multipleDispatch.hpp>
#include "tree/Tree.h"
#include "tree/TreeFeatures.h"

#include "alphabet/RankedSymbol.h"

namespace tree {

/**
 * This class contains methods to print XML representation of tree to the output stream.
 */
class TreeToStringComposer : public std::SingleDispatchFirstStaticParam < void, std::ostream &, TreeBase > {
	static void compose ( std::ostream &, const RankedNode & tree );
	static void compose ( std::ostream &, const alphabet::RankedSymbol & subtreeWildcard, const RankedNode & tree );
	static void compose ( std::ostream &, const UnrankedNode & tree );
	static void compose ( std::ostream &, const alphabet::Symbol & subtreeWildcard, const UnrankedNode & tree );

public:
	static void compose ( std::ostream &, const RankedTree & tree );
	static void compose ( std::ostream &, const RankedPattern & tree );
	static void compose ( std::ostream &, const UnrankedTree & tree );
	static void compose ( std::ostream &, const UnrankedPattern & tree );

	/**
	 * Prints XML representation of String to the output stream.
	 * @param tree String to print
	 * @param out output stream to which print the String
	 */
	static void compose ( std::ostream & output, const Tree & tree );

	static TreeToStringComposer & getInstance ( ) {
		static TreeToStringComposer res;

		return res;
	}

};

} /* namespace tree */

#endif /* TREE_TO_STRING_COMPOSER_H_ */
